// ##########################################################################
// ####                                                                  ####
// ####      WarpEd - A portable text editor for OS/2 Warp and Linux     ####
// ####    ===========================================================   ####
// ####                                                                  ####
// #### console.cc                                                       ####
// ####                                                                  ####
// #### Version 0.80  --  October 06, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 1996  Thomas Dreibholz                             ####
// ####               1997  Molbachweg 7                                 ####
// ####                     51674 Wiehl/Germany                          ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <stdio.h>
#include <strings.h>       

#include "console.h"

#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>
#include <term.h>
#include <unistd.h>
#include <ncurses.h>
#undef clear


// #######################################################################
// ===== Initialisierung =================================================
// #######################################################################




/* Terminal initialisieren */
int OutputTerminalClass::OpenTerminal(ColorPrefs *cprefs)
{
   memcpy(&zColorPrefs,cprefs,sizeof(ColorPrefs));
   
   initscr();
   start_color();
   cbreak();
   noecho();
   raw();
   nonl();
   intrflush(stdscr,FALSE);
   keypad(stdscr,TRUE);

   getmaxyx(stdscr,zHeight,zWidth);
   init_pair(NormalColorPair,zColorPrefs.FrontPen,zColorPrefs.BackPen);
   init_pair(BlockColorPair,zColorPrefs.BlockFrontPen,zColorPrefs.BlockBackPen);
   init_pair(InfoColorPair,zColorPrefs.InfoFrontPen,zColorPrefs.InfoBackPen);
   zColorPrefs.FrontPen      = NormalColorPair;
   zColorPrefs.BlockFrontPen = BlockColorPair;
   zColorPrefs.InfoFrontPen  = InfoColorPair;
   cprefs->FrontPen      = NormalColorPair;
   cprefs->BlockFrontPen = BlockColorPair;
   cprefs->InfoFrontPen  = InfoColorPair;

   zWriteLevel = 0;
   zCursorX    = 0;
   zCursorY    = 0;
   zShowCursor = FALSE;
   Clear();
   return(TRUE);
}


/* Terminal schlieen */
void OutputTerminalClass::CloseTerminal()
{
   Clear();
   endwin();
}


// #######################################################################
// ===== Ausgabe =========================================================
// #######################################################################



// Ausgabe beginnen
void OutputTerminalClass::BeginWriting()
{ 
   zWriteLevel++;
}


// Ausgabe beenden
void OutputTerminalClass::EndWriting()
{ 
   zWriteLevel--;
   if(zWriteLevel == 0) {    
      move(zCursorY,zCursorX);
      if(zShowCursor) {

      }
      refresh();
   }
}
   

/* String-Ausgabe */
void OutputTerminalClass::WriteString(int x, int y, int style,
                                      int frontPen, int backPen,
                                      char *str)
{
   if((frontPen < 0) || (frontPen > 3)) {
   beep();
   frontPen = 1;
   }


   attron(COLOR_PAIR(frontPen));
   move(y,x);
   addstr(str);
   attroff(COLOR_PAIR(frontPen));
}


/* 
Zeile ausgeben
x,y:       Koordinaten
width:     Breite der Zeile
offset:    X-Offset
m1,m2:     Markierung
buffer:    Zeiger auf Buffer
*/
void OutputTerminalClass::WriteLine(int x, int y, int width, int offset, 
                                    int m1, int m2,
                                    char* buffer)
{
   int  i,j,k;
   int  len;
   int  style;
   int  frontPen;
   int  backPen;
   char ctrl[32];  
   char out[256];

   // Keine Markierung, wenn beide Positionen gleich sind
   if(m1 == m2) {
      m1 = -1;        
      m2 = -1;
   }

   // Ausgabe auf Offset beginnen: Anfang abschneiden bzw. leerer String
   len = strlen(buffer);
   if(len < offset) {
      ctrl[15] = 0x00;
      buffer   = &ctrl[15];
      len      = 0;
   }
   else {
      buffer += offset;
      len    -= offset;
   }
   
   // Markierung beginnt direkt am Anfang
   if((m1 != -1) && (m1 < offset)) {
      style    = zColorPrefs.BlockStyle;
      frontPen = zColorPrefs.BlockFrontPen; 
      backPen  = zColorPrefs.BlockBackPen;
   } 
   else {
      style    = zColorPrefs.Style;
      frontPen = zColorPrefs.FrontPen;
      backPen  = zColorPrefs.BackPen;
   }
   
   for(i = 0,j = 0,k = offset;i  < width;i++,j++,k++) {
      out[j] = 0x00;
      // Markierung beginnen
      if(k == m1) {
         WriteString(x + i - j, y, style, frontPen, backPen, out);
         style    = zColorPrefs.BlockStyle;
         frontPen = zColorPrefs.BlockFrontPen;
         backPen  = zColorPrefs.BlockBackPen;
         j = 0;
      }
      // Markierung beenden
      else if(k == m2) {
         WriteString(x + i - j, y, style, frontPen, backPen, out);
         style    = zColorPrefs.Style;
         frontPen = zColorPrefs.FrontPen;
         backPen  = zColorPrefs.BackPen;
         j = 0;
      }

      // Zeilenende mit Leerzeichen auffllen
      if(i < len) {
         out[j] = buffer[i];
      }
      else {
         out[j] = ' ';
      }
   }
   out[j] = 0x00;
   WriteString(x + i - j,y,zColorPrefs.Style,frontPen,backPen,out);
}


/* Bildschirm lschen */
void OutputTerminalClass::Clear()
{
   erase();
}


// Sound-Ausgabe
void OutputTerminalClass::Bell()
{ 
   beep();
}


// #######################################################################
// ===== Cursor-Verwaltung ===============================================
// #######################################################################


// Cursor setzen
void OutputTerminalClass::ShowSysCursor(int x, int y)
{
   zCursorX    = x;
   zCursorY    = y;
   zShowCursor = TRUE;
   if(zWriteLevel == 0) {
      move(zCursorY,zCursorX);
      refresh();
   }
}


// Cursor ausschalten
void OutputTerminalClass::HideSysCursor()
{
   move(1024,1024);
   refresh();
}


// #######################################################################
// ===== Eingabe =========================================================
// #######################################################################


/* Zeichen lesen */
int OutputTerminalClass::WaitForKey(int secs, int usecs)
{
   timeout((secs * 1000) + (usecs / 1000));
   return(getch());
}
