// ##########################################################################
// ####                                                                  ####
// ####      WarpEd - A portable text editor for OS/2 Warp and Linux     ####
// ####    ===========================================================   ####
// ####                                                                  ####
// #### lines.cc                                                         ####
// ####                                                                  ####
// #### Version 0.80  --  October 06, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 1996  Thomas Dreibholz                             ####
// ####               1997  Molbachweg 7                                 ####
// ####                     51674 Wiehl/Germany                          ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <bool.h>
#include <stdlib.h>

#include "lines.h"
#include "str.h"


// Standard-Konstruktor
LineManager::LineManager()
{
   isGood = false;
}


/* Zeilenmanager initialisieren */
void LineManager::init(int mr, int mc)
{
   maxRows = mr;
   maxCols = mc;
   count   = 0;
   row     = NULL;
   isGood  = false;
 
   // Speicher fr Array belegen
   row = new char*[maxRows];
   if(row == NULL) return;

   // Zeilenspeicher initialisieren
   for(int i = 0;i < maxRows;i++) {
      row[i] = NULL;
   }

   isGood = true;
}


/* Zeilenmanager freigeben */
LineManager::~LineManager()
{
   if(row) {
      // Zeilenspeicher initialisieren
      for(int i = 0;i < count;i++) {
         if(row[i] != NULL) {
            delete(row[i]);
         }
         row[i] = NULL;
      }
      delete row;
      row = NULL;
   }
}


/* Zeile mit Text belegen */
void LineManager::SetLine(int number, char* text)
{
   if(number < maxRows) {
      // Zeile kopieren
      char* newRow = new char[strlen(text) + 1];
      if(newRow != NULL) {
         strcpy(newRow,text);
      }
      Optimize(newRow);
       
      // Alte Zeile freigeben und neue Zeile eintragen
      if(row[number] != NULL) {
         delete(row[number]);
      }
      row[number] = newRow;
      
      // Anzahl korrigieren
      if(number > count) {
         count = number;
      }
   }
}


/* Zeile abrufen */
void LineManager::GetLine(int number, char *text)
{
   if((number < maxRows) && (row[number] != NULL)) {        
      strcpy(text,row[number]); 
   }
   else {
      text[0] = 0x00;
   }
}


/* Zeilenlnge ermitteln */
int LineManager::GetLineLength(int number)
{
   if((number >= maxRows) || (row[number] == NULL)) {
      return(0);
   }
   return(strlen(row[number]));
}


/* Zeile einfgen */
int LineManager::InsertLine(int number)
{
   if(number <= maxRows - 1) {
      // Fehler, falls Textspeicher voll...      
      if( (count == maxRows - 1) && 
          ((row[maxRows - 1] == NULL) || (strcmp(row[maxRows - 1],""))) ) {
         return(false);
      }
   
      // Restliche Zeilen nach hinten schieben
      for(int i = count;i > number;i--) {
         row[i] = row[i - 1];
      }
      row[number] = NULL;

      // Zhler korrigieren
      if(number > count)
         count = number;
      else
         count++;
      if(count >= maxRows - 1) {
         count = maxRows - 1;
      }
      return(true);
   }
   return(false);
}


/* Zeile lschen */
void LineManager::DeleteLine(int number)
{
   // Zeile lschen
   char* r = row[number];
   for(int i = number;i < count;i++) {
      row[i] = row[i + 1];
   }
   row[count] = NULL;

   // Speicher freigeben
   if(r != NULL) {
      delete r;
   }

   // Zhler korrigieren
   count--;
}


/* Gesamten Text lschen */
void LineManager::DeleteAll()
{
   for(int i = 0;i <= count;i++) {
      if(row[i] != NULL) {
         delete row[i];
      }
      row[i] = NULL;
   }
   count = 0;
}
